module Types.WebEnv
  ( HasWebEnv (..),
    WebEnv (..),
  )
where

import ClassyPrelude
import Network.Wai (Request, Response, ResponseReceived)
import Types.AppTypes
  ( Env (..),
    HasConnectedClientState (getConnectedClientState),
  )
import Types.RoomsState (HasRoomsState (getRoomsState))

class HasWebEnv a where
  getRequest :: a -> Request
  getRespond :: a -> (Response -> IO ResponseReceived)

data WebEnv = WebEnv
  { appEnv :: Env,
    request :: Request,
    respond :: Response -> IO ResponseReceived
  }

instance HasConnectedClientState WebEnv where
  getConnectedClientState = connectedClientsState . appEnv

instance HasRoomsState WebEnv where
  getRoomsState = roomsState . appEnv

instance HasWebEnv WebEnv where
  getRequest = request
  getRespond = respond
